// ObserverSwarm.java 

import swarm.Globals;
import swarm.Selector;

import swarm.defobj.Zone;
import swarm.defobj.ZoneImpl;
import swarm.defobj.Symbol;

import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;

import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.collections.ListImpl;

import swarm.analysis.EZGraph;
import swarm.analysis.EZGraphImpl;

/**
 * ObserverSwarm.java The observer swarm is collection of objects that
 * are used to run and observe the ModelSwarm that actually comprises
 * the simulation.
 *  
 *
 * @author  Marco Agagliate, Antonio de Ruvo
 */
public class ObserverSwarm extends GUISwarmImpl
        {
        // Declare the display parameters and their default values.
        /** Update frequency. */
        public int displayFrequency          = 1;
        /** To stop simulation at the end of a day. */
        public int stopAtDayNumber           = 90;           
        /** To save on a file the price data. */
        public int savePriceData             = 0;
        /** To create file and represent previous day mean price. */
        public int displayPreviousDayMean    = 0;
        /** To show book graph. */
        public int showBookLogGraph             = 0;
        /** To save book data. */
        public int saveBookLogData              = 0;
        /** To show agent's wealth graph. */
        public int showAgentWealthGraph      = 0;
        /** To save agent's wealth data.*/
        public int saveAgentWealthData       = 0;
        /** To show spread graph between first bid and first ask price. */
        public int showBidAskSpreadGraph           = 0;
        /** To save spread data between first bid and first ask price. */
        public int saveBidAskSpreadData            = 0;
        /** To show spread graph between quantities in sell and buy side. */
        public int showBuySellQuantitySpreadGraph           = 0;
        /** To save spread data between quantities in sell and buy side. */
        public int saveBuySellQuantitySpreadData            = 0;
        /** To show spread graph between the first and the last price in the two side. */
        public int showBuySellFirstLastSpreadGraph           = 0;
        /** To save spread data between the first and the last price in the two side. */
        public int saveBuySellFirstLastSpreadData            = 0;
        /** To show the number of shares in the two side before opening or closing. */
        public int showBuySellSharesNumberInOpeningOrClosingAuctions           = 0;

        // Declare other variables local to ObserverSwarm.
        /** the ModelSwarm we are observing */
        public ModelSwarm modelSwarm;
        /** the single Schedule instance */
        public ScheduleImpl displaySchedule;
        /** The book of the simulation. */
        public Book theBook;
        /** two ActionGroup for sequence of GUI events */
        public ActionGroupImpl displayActions;
        /** our graphics or EZGraph output to files*/
        public EZGraphImpl priceGraph, priceFile, bookLogGraph,
                           agentWealth, agentWealthGraph,
                           sharesInSellSideFile, sharesInBuySideFile,
                           minWealthAllFile, meanWealthAllFile,maxWealthAllFile, 
                           minWealthRandomFile,meanWealthRandomFile, maxWealthRandomFile,
                           minWealthMarketImitatingFile,meanWealthMarketImitatingFile,maxWealthMarketImitatingFile,
                           minWealthLocallyImitatingFile,meanWealthLocallyImitatingFile,maxWealthLocallyImitatingFile,
                           minWealthStopLossFile,meanWealthStopLossFile,maxWealthStopLossFile,
                           meanPriceFile, bidAskSpreadGraph,
                           currentBidAskSpreadFile, sharesInBuySideInOpeningOrClosingAuctionsFile,
                           sharesInSellSideInOpeningOrClosingAuctionsFile, quantitySpreadGraph,
                           currentQuantitySpreadFile, firstLastSpreadGraph,
                           currentBuyFirstLastSpreadFile, currentSellFirstLastSpreadFile;
        
        /** Constructor for a new ObserverSwarm. */ 
        public ObserverSwarm(Zone aZone)
                {
                // Use the parent class to create an observer swarm.
                super(aZone);

                // Build a custom probe map.  Without a probe map, the default
                // is to show all variables and messages.  Here we choose to
                // customize the appearance of the probe, giving a nicer
                // interface.

                // Create the probe map and give it the ObserverSwarm class.
                EmptyProbeMapImpl probeMap = new EmptyProbeMapImpl(aZone, getClass());
   
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("displayFrequency",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("stopAtDayNumber",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("displayPreviousDayMean",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("savePriceData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showBookLogGraph",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("saveBookLogData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showAgentWealthGraph",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("saveAgentWealthData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showBidAskSpreadGraph",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("saveBidAskSpreadData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showBuySellQuantitySpreadGraph",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("saveBuySellQuantitySpreadData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showBuySellFirstLastSpreadGraph",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("saveBuySellFirstLastSpreadData",
                                ObserverSwarm.this.getClass()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("showBuySellSharesNumberInOpeningOrClosingAuctions",
                                ObserverSwarm.this.getClass()));
      
                // And finaly install our probe map into the probeLibrary.
                // Note that this library object was automatically created by
                // initSwarm.
                Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
                }

        /** Create the objects used to display the model. */
        public Object buildObjects(){
                // Zone modelZone;
                Selector sel;

                // Use the parent class to initialize the process.
                super.buildObjects();

                // We create the model that we'are actually observing, by
                // creating an instance of the ModelSwarm class, modelSwarm.
                modelSwarm =
                        (ModelSwarm)Globals.env.lispAppArchiver.getWithZone$key(
                                        getZone(), "modelSwarm");

                // Now create probe objects on the model and on ourselves.
          
                 Globals.env.createArchivedProbeDisplay(modelSwarm, "modelSwarm");
                 Globals.env.createArchivedProbeDisplay(this, "observerSwarm");
        
                // Instruct the control panel to wait for a button event: we
                // halt here until someone hits a control panel button.
                // Eventually this will allow the user a chance to fill in
                // parameters before the simulation runs.
                getControlPanel().setStateStopped();
        
                // Allow the model swarm to build its objects.
                modelSwarm.buildObjects();

                // checking the consistence of the displayFrequency with the istantNumber:
                if(displayFrequency !=1 && displayFrequency %
                        modelSwarm.istantNumber != 0)
                        {
                        System.out.println("displayFrequency must be 1 or multiple of istantNumber.\n");
                        System.exit(1);
                        }

                theBook = modelSwarm.getBook();
         
                priceGraph = new EZGraphImpl(getZone (), "Current price",
                                      "Ticks x days.", "Price.", "priceGraph");
                Globals.env.setWindowGeometryRecordName (priceGraph, "priceGraph");

                // the data for the priceGraph
                priceGraph.createSequence$withFeedFrom$andSelector
                          ("Current price", theBook,
                          SwarmUtils.getSelector(theBook,"getPrice"));

                if (displayPreviousDayMean==1){
                   priceGraph.createSequence$withFeedFrom$andSelector
                                ("Day-1 mean p.", theBook,
                                SwarmUtils.getSelector(theBook,"getMeanPrice"));
                }

	        if (savePriceData==1){
                  // file
                  priceFile =  new EZGraphImpl(getZone(), "price");
                        priceFile.createSequence$withFeedFrom$andSelector
                        ("txt", theBook, SwarmUtils.getSelector(theBook, "getPrice"));

                  // file
                   meanPriceFile =  new EZGraphImpl(getZone(), "meanPrice");
                        meanPriceFile.createSequence$withFeedFrom$andSelector
                        ("txt", theBook, SwarmUtils.getSelector(theBook, "getMeanPrice"));     
	        }

                if (showBookLogGraph==1 || showBuySellSharesNumberInOpeningOrClosingAuctions==1){
                  bookLogGraph = new EZGraphImpl(getZone (), "Book log", "Tiks x days.",
                                            "Number of shares logged in buy or sell side.",
                                            "bookLogGraph");
                  Globals.env.setWindowGeometryRecordName (bookLogGraph, "bookLogGraph");

                  // the data for the bookLogGraph
		  if (showBookLogGraph==1){
                    bookLogGraph.createSequence$withFeedFrom$andSelector
                          ("Shares in sell side", theBook,
                          SwarmUtils.getSelector(theBook,"getSharesInSellSide"));
                    bookLogGraph.createSequence$withFeedFrom$andSelector
                          ("Shares in buy side", theBook,
                          SwarmUtils.getSelector(theBook,"getSharesInBuySide"));
		  }
		  if(showBuySellSharesNumberInOpeningOrClosingAuctions==1){
		    bookLogGraph.createSequence$withFeedFrom$andSelector
                          ("Shares in sell side before opening or closing", theBook,
                          SwarmUtils.getSelector(theBook,"getSharesInSellSideInOpeningOrClosingAuctions"));
		    bookLogGraph.createSequence$withFeedFrom$andSelector
                          ("Shares in buy side before opening or closing", theBook,
                          SwarmUtils.getSelector(theBook,"getSharesInBuySideInOpeningOrClosingAuctions")); 
	          } 
	        }

		if (saveBookLogData==1){
                  // file
                  sharesInSellSideFile =  new EZGraphImpl(getZone(), "sharesInSellSide");
                  sharesInSellSideFile.createSequence$withFeedFrom$andSelector
                                            ("txt", theBook, SwarmUtils.getSelector(theBook,
                                             "getSharesInSellSide"));
                  sharesInBuySideFile =  new EZGraphImpl(getZone(), "sharesInBuySide");
                  sharesInBuySideFile.createSequence$withFeedFrom$andSelector
                                            ("txt", theBook, SwarmUtils.getSelector(theBook,
                                             "getSharesInBuySide"));
                  sharesInSellSideInOpeningOrClosingAuctionsFile =  new EZGraphImpl(getZone(),
                                                       "sharesInSellSideInOpeningOrClosingAuctions");
                  sharesInSellSideInOpeningOrClosingAuctionsFile.createSequence$withFeedFrom$andSelector
                                            ("txt", theBook, SwarmUtils.getSelector(theBook,
                                             "getSharesInSellSideInOpeningOrClosingAuctions"));
                  sharesInBuySideInOpeningOrClosingAuctionsFile =  new EZGraphImpl(getZone(),
                                                        "sharesInBuySideInOpeningOrClosingAuctions");
                  sharesInBuySideInOpeningOrClosingAuctionsFile.createSequence$withFeedFrom$andSelector
                                            ("txt", theBook, SwarmUtils.getSelector(theBook,
                                             "getSharesInBuySideInOpeningOrClosingAuctions"));
	        }

		if (showBidAskSpreadGraph==1){
		  bidAskSpreadGraph = new EZGraphImpl(getZone(), "Current bid-ask spread",
			                        "Tiks x days.", "Spread", "bidAskSpreadGraph");
                  Globals.env.setWindowGeometryRecordName (bidAskSpreadGraph,
                                                           "bidAskSpreadGraph");

		  // the data for bidAskSpreadGraph
                  bidAskSpreadGraph.createSequence$withFeedFrom$andSelector
                        ("Bid-ask spread", theBook,
                  SwarmUtils.getSelector(theBook, "getSpread"));
		}

		if (saveBidAskSpreadData==1){
		  //file
		    currentBidAskSpreadFile = new EZGraphImpl(getZone(), "bid-askSpread");
		    currentBidAskSpreadFile.createSequence$withFeedFrom$andSelector("txt", 
                                        theBook, SwarmUtils.getSelector(theBook, "getSpread"));
		}

		if (showBuySellQuantitySpreadGraph==1){
		  quantitySpreadGraph = new EZGraphImpl(getZone(),
                                             "Current buy and sell quantity spread",
                                             "Tiks x days.", "Spread", "quantitySpreadGraph");
                  Globals.env.setWindowGeometryRecordName (quantitySpreadGraph,
                                                           "quantitySpreadGraph");

		  // the data for quantitySpreadGraph
                  quantitySpreadGraph.createSequence$withFeedFrom$andSelector
                        ("Quantity spread", theBook,
                  SwarmUtils.getSelector(theBook, "getBuySellQuantitySpread"));
		}

		if (saveBuySellQuantitySpreadData==1){
		  //file
		    currentQuantitySpreadFile = new EZGraphImpl(getZone(), "quantitySpread");
		    currentQuantitySpreadFile.createSequence$withFeedFrom$andSelector("txt", 
                         theBook, SwarmUtils.getSelector(theBook, "getBuySellQuantitySpread"));
		}

		if (showBuySellFirstLastSpreadGraph==1){
		  firstLastSpreadGraph = new EZGraphImpl(getZone(),
		                           "Current first-last spread", "Ticks x days.",
                                           "Spread", "firstLastSpreadGraph");
                  Globals.env.setWindowGeometryRecordName (firstLastSpreadGraph,
                                                           "firstLastSpreadGraph");

		  // the data for firstLastSpreadGraph
                  firstLastSpreadGraph.createSequence$withFeedFrom$andSelector
                        ("Buy first-last spread", theBook,
                  SwarmUtils.getSelector(theBook, "getBuyFirstLastSpread"));
                  firstLastSpreadGraph.createSequence$withFeedFrom$andSelector
                        ("Sell first-last spread", theBook,
                  SwarmUtils.getSelector(theBook, "getSellFirstLastSpread"));
		}

		if (saveBuySellFirstLastSpreadData==1){
		 //file
		 currentBuyFirstLastSpreadFile = new EZGraphImpl(getZone(), 
                                                "buyFirstLastSpread");
		 currentBuyFirstLastSpreadFile.createSequence$withFeedFrom$andSelector("txt", 
                        theBook, SwarmUtils.getSelector(theBook, "getBuyFirstLastSpread"));
	         currentSellFirstLastSpreadFile = new EZGraphImpl(getZone(), 
                                                "sellFirstLastSpread");
	         currentSellFirstLastSpreadFile.createSequence$withFeedFrom$andSelector("txt", 
                        theBook, SwarmUtils.getSelector(theBook, "getSellFirstLastSpread"));
		}

                if (showAgentWealthGraph==1){
                  agentWealthGraph = new EZGraphImpl(getZone(), "Agent's wealth",
                                        "Ticks x days.", "Wealth", "agentWealthGraph");
                  Globals.env.setWindowGeometryRecordName (agentWealthGraph,
                                         "agentWealthGraph");

                  // the data for the agentGraph
                  agentWealthGraph.createMinSequence$withFeedFrom$andSelector
                        ("Min Wealth (all)", modelSwarm.getAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createAverageSequence$withFeedFrom$andSelector
                        ("Mean Wealth (all)", modelSwarm.getAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMaxSequence$withFeedFrom$andSelector
                        ("Max Wealth (all)", modelSwarm.getAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMinSequence$withFeedFrom$andSelector
                        ("Min Wealth (r.)", modelSwarm.getRandomAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createAverageSequence$withFeedFrom$andSelector
                        ("Mean Wealth (r.)", modelSwarm.getRandomAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMaxSequence$withFeedFrom$andSelector
                        ("Max Wealth (r.)", modelSwarm.getRandomAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMinSequence$withFeedFrom$andSelector
                        ("Min Wealth (m.i)", modelSwarm.getMarketImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createAverageSequence$withFeedFrom$andSelector
                        ("Mean Wealth (m.i)", modelSwarm.getMarketImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMaxSequence$withFeedFrom$andSelector
                        ("Max Wealth (m.i)", modelSwarm.getMarketImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMinSequence$withFeedFrom$andSelector
                        ("Min Wealth (l.i)", modelSwarm.getLocallyImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createAverageSequence$withFeedFrom$andSelector
                        ("Mean Wealth (l.i)", modelSwarm.getLocallyImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMaxSequence$withFeedFrom$andSelector
                        ("Max Wealth (l.i)", modelSwarm.getLocallyImitatingAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMinSequence$withFeedFrom$andSelector
                        ("Min Wealth (s.l)", modelSwarm.getStopLossAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createAverageSequence$withFeedFrom$andSelector
                        ("Mean Wealth (s.l)", modelSwarm.getStopLossAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  agentWealthGraph.createMaxSequence$withFeedFrom$andSelector
                        ("Max Wealth (s.l)", modelSwarm.getStopLossAgentList(),
                        SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
	        }

                if (saveAgentWealthData==1){
                  // file
                  minWealthAllFile =  new EZGraphImpl(getZone(), "minWealthAll");
                        minWealthAllFile.createMinSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  meanWealthAllFile =  new EZGraphImpl(getZone(), "meanWealthAll");
                        meanWealthAllFile.createAverageSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  maxWealthAllFile =  new EZGraphImpl(getZone(), "maxWealthAll");
                        maxWealthAllFile.createMaxSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  minWealthRandomFile =  new EZGraphImpl(getZone(), "minWealthRandom");
                        minWealthRandomFile.createMinSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getRandomAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  meanWealthRandomFile =  new EZGraphImpl(getZone(), "meanWealthRandom");
                        meanWealthRandomFile.createAverageSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getRandomAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  maxWealthRandomFile =  new EZGraphImpl(getZone(), "maxWealthRandom");
                        maxWealthRandomFile.createMaxSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getRandomAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  minWealthMarketImitatingFile =  new EZGraphImpl(getZone(), "minWealthMarketImitating");
                        minWealthMarketImitatingFile.createMinSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getMarketImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  meanWealthMarketImitatingFile =  new EZGraphImpl(getZone(), "meanWealthMarketImitating");
                        meanWealthMarketImitatingFile.createAverageSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getMarketImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  maxWealthMarketImitatingFile =  new EZGraphImpl(getZone(), "maxWealthMarketImitating");
                        maxWealthMarketImitatingFile.createMaxSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getMarketImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  minWealthLocallyImitatingFile =  new EZGraphImpl(getZone(), "minWealthLocallyImitating");
                        minWealthLocallyImitatingFile.createMinSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getLocallyImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  meanWealthLocallyImitatingFile =  new EZGraphImpl(getZone(), "meanWealthLocallyImitating");
                        meanWealthLocallyImitatingFile.createAverageSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getLocallyImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  maxWealthLocallyImitatingFile =  new EZGraphImpl(getZone(), "maxWealthLocallyImitating");
                        maxWealthLocallyImitatingFile.createMaxSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getLocallyImitatingAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  minWealthStopLossFile =  new EZGraphImpl(getZone(), "minWealthStopLoss");
                        minWealthStopLossFile.createMinSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getStopLossAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  meanWealthStopLossFile =  new EZGraphImpl(getZone(), "meanWealthStopLoss");
                        meanWealthStopLossFile.createAverageSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getStopLossAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
                  maxWealthStopLossFile =  new EZGraphImpl(getZone(), "maxWealthStopLoss");
                        maxWealthStopLossFile.createMaxSequence$withFeedFrom$andSelector
                        ("txt", modelSwarm.getStopLossAgentList(),
                                SwarmUtils.getSelector("BasicSumAgent","getWealthAtMeanDailyPrice"));
	        }
        return this;
	}             

        /**
         * Create the actions necessary for the simulation. This is where
         * the schedule is built (but not run!)  Here we create a display
         * schedule - this is used to display the state of the world and
         * check for user input. 
         */
        public Object buildActions()
                {
                Selector sel;
                
                // Use the parent class to begin the process.
                super.buildActions();

                // Call on the model swarm to build and schedule its actions.
                modelSwarm.buildActions();

                // Create an ActionGroup for display.  This is a list of
                // display actions that we want to occur at each step in
                // simulation time. 
                // "doTkEvents", is required at the end to make everything 
                // happen. We then check to see if modelSwarm has
                // told us to stop the simulation.
                displayActions = new ActionGroupImpl(getZone());

                // to update probes
                sel = SwarmUtils.getSelector(Globals.env.probeDisplayManager, "update");
                displayActions.createActionTo$message(Globals.env.probeDisplayManager, sel);
                sel = SwarmUtils.getSelector(getActionCache(), "doTkEvents");
                displayActions.createActionTo$message(getActionCache(), sel);

                sel = SwarmUtils.getSelector(priceGraph, "step");
                displayActions.createActionTo$message(priceGraph, sel);

                if (savePriceData==1)
                        {
                        sel = SwarmUtils.getSelector(priceFile, "step");
                        displayActions.createActionTo$message(priceFile, sel);
                                
                        sel = SwarmUtils.getSelector(meanPriceFile, "step");
                        displayActions.createActionTo$message(meanPriceFile, sel);
                                
                        }

                if (showBookLogGraph==1 || showBuySellSharesNumberInOpeningOrClosingAuctions==1 )
                        {
                        sel = SwarmUtils.getSelector(bookLogGraph, "step");
                        displayActions.createActionTo$message(bookLogGraph, sel);
                        }

                if (saveBookLogData==1)
                        {
                        sel = SwarmUtils.getSelector(sharesInSellSideFile, "step");
                        displayActions.createActionTo$message(sharesInSellSideFile, sel);
                        sel = SwarmUtils.getSelector(sharesInBuySideFile, "step");
                        displayActions.createActionTo$message(sharesInBuySideFile, sel);
                        sel = SwarmUtils.getSelector(sharesInSellSideInOpeningOrClosingAuctionsFile,
                                                     "step");
                        displayActions.createActionTo$message(
                                                 sharesInSellSideInOpeningOrClosingAuctionsFile, sel);
                        sel = SwarmUtils.getSelector(sharesInBuySideInOpeningOrClosingAuctionsFile,
                                                     "step");
                        displayActions.createActionTo$message(
                                                  sharesInBuySideInOpeningOrClosingAuctionsFile, sel);
                        }

                if (showBidAskSpreadGraph==1)
                        {
                        sel = SwarmUtils.getSelector(bidAskSpreadGraph, "step");
                        displayActions.createActionTo$message(bidAskSpreadGraph, sel);
                        }

                if (saveBidAskSpreadData==1)
                        {
                        sel = SwarmUtils.getSelector(currentBidAskSpreadFile, "step");
                        displayActions.createActionTo$message(currentBidAskSpreadFile, sel);
                        }

                if (showBuySellQuantitySpreadGraph==1)
                        {
                        sel = SwarmUtils.getSelector(quantitySpreadGraph, "step");
                        displayActions.createActionTo$message(quantitySpreadGraph, sel);
                        }

                if (saveBuySellQuantitySpreadData==1)
                        {
                        sel = SwarmUtils.getSelector(currentQuantitySpreadFile, "step");
                        displayActions.createActionTo$message(currentQuantitySpreadFile, sel);
                        }

		if (showBuySellFirstLastSpreadGraph==1)
		        {
                        sel = SwarmUtils.getSelector(firstLastSpreadGraph, "step");
                        displayActions.createActionTo$message(firstLastSpreadGraph, sel);
			}

		if (saveBuySellFirstLastSpreadData==1)
		    {
                    sel = SwarmUtils.getSelector(currentBuyFirstLastSpreadFile, "step");
                    displayActions.createActionTo$message(currentBuyFirstLastSpreadFile, sel);
                    sel = SwarmUtils.getSelector(currentSellFirstLastSpreadFile, "step");
                    displayActions.createActionTo$message(currentSellFirstLastSpreadFile, sel);
	            }

                if (showAgentWealthGraph==1)
                        {
                        sel = SwarmUtils.getSelector(agentWealthGraph, "step");
                        displayActions.createActionTo$message(agentWealthGraph, sel);
                        }

                if (saveAgentWealthData==1)
                        {
                        sel = SwarmUtils.getSelector(minWealthAllFile, "step");
                        displayActions.createActionTo$message(minWealthAllFile, sel);
                        sel = SwarmUtils.getSelector(meanWealthAllFile, "step");
                        displayActions.createActionTo$message(meanWealthAllFile, sel);
                        sel = SwarmUtils.getSelector(maxWealthAllFile, "step");
                        displayActions.createActionTo$message(maxWealthAllFile, sel);
                        sel = SwarmUtils.getSelector(minWealthRandomFile, "step");
                        displayActions.createActionTo$message(minWealthRandomFile, sel);
                        sel = SwarmUtils.getSelector(meanWealthRandomFile, "step");
                        displayActions.createActionTo$message(meanWealthRandomFile, sel);
                        sel = SwarmUtils.getSelector(maxWealthRandomFile, "step");
                        displayActions.createActionTo$message(maxWealthRandomFile, sel);
                        sel = SwarmUtils.getSelector(minWealthMarketImitatingFile, "step");
                        displayActions.createActionTo$message(minWealthMarketImitatingFile, sel);
                        sel = SwarmUtils.getSelector(meanWealthMarketImitatingFile, "step");
                        displayActions.createActionTo$message(meanWealthMarketImitatingFile, sel);
                        sel = SwarmUtils.getSelector(maxWealthMarketImitatingFile, "step");
                        displayActions.createActionTo$message(maxWealthMarketImitatingFile, sel);
                        sel = SwarmUtils.getSelector(minWealthLocallyImitatingFile, "step");
                        displayActions.createActionTo$message(minWealthLocallyImitatingFile, sel);
                        sel = SwarmUtils.getSelector(meanWealthLocallyImitatingFile, "step");
                        displayActions.createActionTo$message(meanWealthLocallyImitatingFile, sel);
                        sel = SwarmUtils.getSelector(maxWealthLocallyImitatingFile, "step");
                        displayActions.createActionTo$message(maxWealthLocallyImitatingFile, sel);
                        sel = SwarmUtils.getSelector(minWealthStopLossFile, "step");
                        displayActions.createActionTo$message(minWealthStopLossFile, sel);
                        sel = SwarmUtils.getSelector(meanWealthStopLossFile, "step");
                        displayActions.createActionTo$message(meanWealthStopLossFile, sel);
                        sel = SwarmUtils.getSelector(maxWealthStopLossFile, "step");
                        displayActions.createActionTo$message(maxWealthStopLossFile, sel);
                        }

                sel = SwarmUtils.getSelector(this, "checkToStop");
                displayActions.createActionTo$message(this, sel);

                // Finally, put the ActionGroup into a display schedule.
                // Note that the repeat interval is set by our
                // own Swarm data structure. The display is frequently the slowest part of a
                // simulation, when it is redraw less frequently things are faster
                // We can use here a display frequency lower (e.g.1) than the number of steps
                // in the model (step number = agentNumber) to display the prices of each
                // step-tick
                displaySchedule = new ScheduleImpl(getZone(), displayFrequency);
                displaySchedule.at$createAction(displayFrequency-1, displayActions);
          
                return this;
                }
    
       /**
        *  Activate the schedules so that they are ready to run.  The
        *  swarmContext argument is the zone in which the ObserverSwarm is
        *  activated.  Typically the ObserverSwarm is the top-level swarm,
        *  so it is activated in "null".  The other (sub)swarms and
        *  schedules will be activated inside of the ObserverSwarm
        *  context.
        */
        public Activity activateIn(Swarm swarmContext)
                {
                // Use the parent class to activate ourselves in the context
                // passed to us.
                super.activateIn(swarmContext);

                // Now activate the model swarm in the ObserverSwarm context.
                modelSwarm.activateIn(this);

                // Then activate the ObserverSwarm schedule in the
                // ObserverSwarm context.
                displaySchedule.activateIn(this);

                // Finally, return the activity we have built - the thing that
                // is ready to run.
                return getActivity();
                }

        /** To check for the stopping conditions. */
        public void checkToStop()
                {
                // if stopAtDayNumber is left to 0, the program will never stop
                // this stop occurs after the first tick of the time in modelSwarm,
                // but this fact does not affect the results in the observerSwarm
                // being the stop performed before the graph update
                if (stopAtDayNumber !=0 && stopAtDayNumber <= modelSwarm.getCurrentDay())
                        {
                        System.out.println("Stopping at day number" + modelSwarm.getCurrentDay());
                        // we can restart by pressing "Start" or "Next", but the simulation
                        // will run for displayFrequency ticks and then it will stop again
                        getControlPanel().setStateStopped();
                        }
                }
        }
